//--------------------------------------------------------------------------------------
// Mains AC Non Invasive 3 + Ethernet Example
// Last revision 30 November 2010
// Licence: GNU GPL openenergymonitor.org
// By Trystan Lea
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
// ETHERNET
//--------------------------------------------------------------------------------------
#include "etherShield.h"
byte mac[] = {0x54,0x52,0x58,0x10,0x00,0x18};         //Ethernet shield mac address
byte ip[] = {192,168,1,8};                            //Ethernet shield ip address
byte gateway[] = {192,168,1,1};                       //Gateway ip
byte server[] = {85, 92, 86, 84};                     //Server ip
#define port 80
char str[250];

//----------------------------------------------------------------------------
// Load Energy Monitor library and create new instance
//----------------------------------------------------------------------------
#include "Emon.h"    //Load the library
EnergyMonitor emon;  //Create an instance

double whInc;                                          //energy calc from realPower
unsigned long tmillis, lmillis, lastupdate;

//--------------------------------------------------------------------------------------
// SETUP
//--------------------------------------------------------------------------------------
void setup()
{
  emon.setPins(0,1);                                 //Energy monitor analog pins
  emon.calibration( 1.116111611, 0.128401361, 2.3);  //Energy monitor calibration
  
  Serial.begin(9600);
  
  client_setup(mac,ip,gateway,server);    //Setup ethernet client
  client_timeout(5,1000);                 //Set timeout variables
}

//--------------------------------------------------------------------------------------
// MAIN LOOP
//--------------------------------------------------------------------------------------
void loop()
{
  if ((millis()-lastupdate)>10000)  //Every 10 seconds
  { lastupdate = millis();
    
    emon.calc(20,2000);              //Energy Monitor calc function

    //---------------------------------------------------------------------------------------------
    // wh increment calculation from realPower value
    //---------------------------------------------------------------------------------------------
    lmillis = tmillis;                                       //timing to determine amount of time since last call
    tmillis= millis();                                       
    whInc += (emon.realPower * (tmillis-lmillis)) / 3600000.0;    //energy calc
   
    Serial.print(emon.realPower);    
    Serial.print(' ');  
    Serial.println(whInc);  
    
    createJSON();
    sendEthernet();
  }
}

//--------------------------------------------------------------------------------------
// Construct JSON string
//--------------------------------------------------------------------------------------
void createJSON()
{
  strcpy(str,"/post.php?json=");                //URL
                                                //If your using a shared server add in full URL here
  
  srtJSON(str);                                 //Start JSON string
  addJSON(str,"realPower",  emon.realPower);    //Add a variable
  addJSON(str,"wh",  whInc);    //Add a variable
  endJSON(str);                                 //End JSON string
}

//--------------------------------------------------------------------------------------
// Send the string to the server
//--------------------------------------------------------------------------------------
void sendEthernet()
{
  if (ethernet_send("",str))                     //Try to send the string NOTE: success on recieve of "ok" from server
  { Serial.println("Data sent");} 
  else
  { Serial.println("Failed to send"); }
}
